@@ -1,4 +1,5 @@
-.check <- function(x, w, na.rm = FALSE, check_NA = TRUE)
+# check data and weight
+.check_data_weights <- function(x, w, na.rm = FALSE, check_NA = TRUE)
 {
     if (missing(w))
         stop("Argument 'w' is missing\n", call. = FALSE)
@@ -11,6 +12,7 @@
     if (n != length(w))
         stop("Data vector and weights are not of the same dimension\n",
 	        call. = FALSE)
+    # empty data
     if (n == 0)
         return(NULL)
 
@@ -39,11 +41,12 @@
     list(x = x, w = w, n = n)
 }
 # check and extract data from survey.design object
-.checkformula <- function(f, design, na.rm = FALSE, check_NA = TRUE)
+.check_formula <- function(f, design, na.rm = FALSE, check_NA = TRUE)
 {
     if (inherits(f, "formula")) {
         if (length(all.vars(f)) > 1)
-            stop("Formula must refer to one variable only\n", call. = FALSE)
+            stop("Formula must refer to one r.h.s. variable only\n",
+                call. = FALSE)
         tf <- stats::terms.formula(f)
         if(attr(tf, "response") != 0)
             stop("The LHS of the formula must not be defined\n", call. = FALSE)
@@ -64,6 +67,10 @@
         }
     }
     w <- as.numeric(1 / design$prob)
+    # special return for empty data
+    if (length(y) == 0)
+        return(list(failure = TRUE))
+
     # check for missing values
     failure <- FALSE
     if (check_NA) {
@@ -82,7 +89,8 @@
     list(failure = failure, yname = yname, y = y, w = w, design = design)
 }
 # check and extract data from survey.design object (for regression)
-.checkreg <- function(formula, design, var = NULL, xwgt = NULL, na.rm = FALSE)
+.check_regression <- function(formula, design, var = NULL, xwgt = NULL,
+    na.rm = FALSE)
 {
     if (!inherits(formula, "formula"))
         stop("Argument '", formula, "' must be a formula\n", call. = FALSE)
@@ -90,7 +98,7 @@
     # heteroscedasticity (only one variable); without NA handling; we will
     # deal with this together with x, y, etc.
     if (!is.null(var))
-        var <- .checkformula(var, design, FALSE, FALSE)$y
+        var <- .check_formula(var, design, FALSE, FALSE)$y
 
     # extract the variables
     mf <- stats::model.frame(formula, design$variables,
@@ -138,42 +146,7 @@
         }
     }
     list(failure = failure, x = x, y = y, var = var, w = w, terms = mt,
-        design = design, xwgt = xwgt)
-}
-# check auxiliary totals and means
-.checkauxiliary <- function(object, data, est = "mean", check.names = TRUE,
-    na.action = stats::na.omit) #FIXME: default: na.fail
-{
-    names_beta <- names(object$estimate)
-    names_data <- names(data)
-    if (is.vector(data))
-        if (NCOL(data) < NROW(data))
-	        data <- t(data)
-
-    # vector of population x-means or -totals
-    if (NROW(data) == 1) {
-        # drop intercept (if there is one)
-        if (object$model$intercept) {
-            names_data <- names_data[-1]
-            names_beta <- names_beta[-1]
-        }
-        if (length(data) != object$model$p)
-            stop("Length of auxiliary data does not match\n", call. = FALSE)
-
-        if (check.names && !is.null(names_data)) {
-            if (!all(names_data == names_beta))
-                stop("Variable names do not match (check.names = TRUE)",
-                    call. = FALSE)
-        }
-        x <- data
-    # compute mean/total based on design matrix
-    } else {
-        mf <- stats::model.frame(object$call$formula, data,
-            na.action = na.action)
-        xmat <- stats::model.matrix(stats::terms(mf), mf)
-        x <- switch(est, "mean" = colMeans(xmat), "total" = colSums(xmat))
-    }
-    unname(x)
+        design = design, xwgt = xwgt, yname = yname)
 }
 # psi functions
 .psi_function <- function(x, k, psi = c("Huber", "Huberasym", "Tukey"))
@@ -196,6 +169,11 @@
         tmp$res
     }
 }
+# weight function
+.psi_wgt_function <- function(x, k, psi = c("Huber", "Huberasym", "Tukey"))
+{
+    .psi_function(x, k, psi) / x
+}
 # onAttach function
 .onAttach <- function(libname, pkgname)
 {
@@ -242,7 +220,7 @@
      88   Y8. .8P 88  dP  \\__ \\ |_| | |   \\ V /  __/ |_| |
      88    'Y8P'  88e8P'  |___/\\__,_|_|    \\_/ \\___|\\__, |
 						     __/ |
-					version 0.2 |___/\n\ntype: package?robsurvey to learn more
+					version 0.3 |___/\n\ntype: package?robsurvey to learn more
 use:  library(robsurvey, quietly = TRUE) to suppress the
       start-up message\n")
    }
