@@ -3,7 +3,7 @@
 #include "regression.h"
 #include "regression_cov.h"
 #include "trimmedwinsorized.h"
-#include "huberm.h"
+#include "huber2.h"
 #include "psifunctions.h"
 
 // create arrays describing each C routine
@@ -11,7 +11,8 @@ static const R_CMethodDef cMethods[]  = {
     {"wtrimmedmean", (DL_FUNC) &wtrimmedmean, 7},
     {"wwinsorizedmean", (DL_FUNC) &wwinsorizedmean, 6},
     {"huberm", (DL_FUNC) &huberm, 11},
-    {"rwlslm", (DL_FUNC) &rwlslm, 17},
+    {"rwlslm", (DL_FUNC) &rwlslm, 19},
+    {"wlslm", (DL_FUNC) &wlslm, 8},
     {"wquantile", (DL_FUNC) &wquantile, 5},
     {"wkwinsorizedmean", (DL_FUNC) &wkwinsorizedmean, 6},
     {"cov_reg_model", (DL_FUNC) &cov_reg_model, 13},
